<?php
/* 
    Plugin Name: PHP My Profiler
    Plugin URI: http://www.php-trivandrum.org/open-php-myprofiler/
    Version: 1.00
    Author: Jiju Thomas Mathew <jijutm@saturn.in>
    Author URI: http://www.freelancer.com/u/jthoma.html
    Description: Plugin to integrate a MySQL Profiler into the Wordpress. Though this has issues that this will be plugged into the wordpress flow after init and that logging is done on shutdown hook, some of the database calls are never logged. Still that is not an issue as mostly slowness would be caused by database calls from the theme and plugins. Note that logging may make system more slow and this is for identifying the bottlenecks and should never left on for long times.
*/

if (!class_exists('wp_php_my_profiler')):

define( 'WP_PHPMYPROFILER_PATH', plugin_dir_path(__FILE__) );

  class wp_php_my_profiler
  {
    // Plugin configuration
    public $name;
    
    public $text_domain;

    public $default_options = array(
        'trigger_ip' => false,
        'post_type' => 'profiler_log'
    );

    private $log;
    
    // Singleton
    private static $inst = null;
    
    public static function Instance($new = false)
    {
      if (self::$inst == null || $new)
      {
        self::$inst = new wp_php_my_profiler();
      }
      return self::$inst;
    }
    
    private function __construct()
    {
      $this->name = strtolower(get_class());
      $this->text_domain = $this->name;
      $this->load_options();
      
      load_plugin_textdomain($this->text_domain, false, dirname(plugin_basename(__FILE__)));

      add_action('plugins_loaded', array(&$this, 'init'));
      add_action('shutdown', array(&$this, 'cleanup'));
    }

    private function load_options()
    {
      foreach($this->default_options as $k => $v)
      {
        $this->default_options[$k] = get_option($this->name . '_' . $k, $v);
      }
    }
    
    public function init(){
        $this->setupLogPath();
        $this->startProfiling();
    }
    
    private function setupLogPath(){
        $uploads = wp_upload_dir();
        $this->log = $uploads['basedir'] . '/' . $this->name . '/' . date("Y-m-d-G").'.log';
        if(!is_dir(dirname($this->log))){
            mkdir(dirname($this->log),0755,true);
        }
    }

    private function startProfiling(){
        global $wpdb;
        $wpdb->query('set profiling_history_size=100');
        $wpdb->query('set profiling=1');
    }
    private function stopProfiling(){
        global $wpdb;
        $wpdb->query('set profiling=0');
    }
    
    private function collectData(){
        $rv = array();
        global $wpdb;
        $rv = $wpdb->get_results('show profiles', 'ARRAY_A');
        if(empty($rv)){
            return false;
        }
        array_walk($rv, array($this, 'getDetails'));
        return $rv;
    }
    
    private function getDetails(&$item, $index){
        global $wpdb;
        $rsv = $wpdb->get_results('select min(seq) seq,state,count(*) numb_ops, '
                . 'round(sum(duration),5) sum_dur, round(avg(duration),5) avg_dur, '
                . 'round(sum(cpu_user),5) sum_cpu, round(avg(cpu_user),5) avg_cpu '
                . 'from information_schema.profiling '
                . 'where query_id = ' . $item['Query_ID']
                . ' group by state order by seq', 'ARRAY_A');
        if($rsv){
            $item['detail'] = $rsv;
        }        
    }
    
    private function create_log(){
        $post = array(
            'comment_status' => 'closed',
            'ping_status' => 'closed',
            'post_content' => $this->log,
            'post_name' => $this->name . '-' . md5($this->log),
            'post_status' => 'private',
            'post_type' => $this->default_options['post_type']
        );
        wp_insert_post($post);
    }
    
    public function cleanup(){
        $this->stopProfiling();
        $data['instance'] = array('timestamp' => time(), 'request' => $_SERVER['REQUEST_URI' ]);
	    ob_start();
        $data['profiles'] = $this->collectData();
	    ob_get_clean();
        if(empty($data['profiles']) or count($data['profiles']) == 0) return;
        $logData = base64_encode(gzcompress(serialize($data))) . "\n";
        if(!file_exists($this->log)){
            file_put_contents($this->log, '#PhpMyProfiler' . "\n");
            $this->create_log();
        }
        file_put_contents($this->log, $logData, FILE_APPEND | LOCK_EX);
        $this->log = false; // dont want to call a second time
    }

    
}
  
wp_php_my_profiler::Instance();
   
endif;
?>
